function [ly,tmp2]=evalfls_s(x,fls,npte)
// ----------------------------------------------------------------------
// Evaluate fls structure using scilab code
// ----------------------------------------------------------------------
// This file is part of sciFLT ( Scilab Fuzzy Logic Toolbox )
// Copyright (C) 2004 Jaime Urzua Grez
// mailto:jaime_urzua@yahoo.com
// Toolbox Revision 0.2 -- 2004/10/12
// ----------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// ----------------------------------------------------------------------
// Notes:
// (1) This routine is builded intially contrast some steps, but the
//     computation routine is developed in fortran.
//     See evalfls.sci for more information
// ----------------------------------------------------------------------

if (argn(2)<2) then
	error("evalfls2 need at least 2 parameters.");
end

// MAKE SOME BASIC PRE-TEST
if (typeof(fls)~="fls") then
	error("The second parameter must be a fls.");
end

if ((fls.type~="m")&(fls.type~="ts")) then
	error("Unknow fls type!");
end

// CHECK RULES
[a,b]=checkrule(fls);
if (a>0) then	
	error("ERROR IN RULES... "+b);
end

// SOME BASIC DECLARATIONS
nrules=size(fls.rule,1);
ninputs=length(fls.input);
noutputs=length(fls.output);
npoints=size(x,1);

if ((nrules*npoints*ninputs*noutputs)==0) then
	error("Nothing to evaluate.!");
end

if (size(x,2)~=ninputs) then
	error("Incorrect size of inputs, need a matrix with "+string(ninputs)+" columns.");
end

// INITIALIZE SOME COMMOND VALUES
if (fls.type=="m") then
	// MAMDANI CASE
	if (argn(2)<3) then
		// TAKE 1001 POINTS TO USE IN DEFUZZIFICATION ALGORITHM
		// THE USER CAN CHANGE THIS WITH npte PARAMETER
		// THIS VALUE AFFECT DIRECTLY THE OUTPUTS VALUES !!!
		// SEE THE EXAMPLE ABOUT DEFUZZIFICATION ALGORITHM
		npte=1001;
	end
	if (size(npte,"*")==1) then
		if ((npte<=0)|((int(npte)-npte)>0)) then
			error("The npte parameter must be a integer(s) greather(s) than 0.");
		end
		npev=ones(noutputs,1)*npte;
	else
		if (size(npte,"*")~=noutputs) then
			error("The size of npte parameter must be equal to number of outputs")
		end
		npev=npte(:);
		nok1=find(npev<=0);
		nok2=find((int(npev)-npev)>0);
		if (nok1|nok2) then
			error("The npte parameter must be a vector of integer(s) greather(s) than 0.");
		end		
	end
end

// ---------------------------------------------
// COMPUTE THE IF PART FOR ALL INPUT PAIRS
// THIS IS VALID WITH MAMDANI AND TAKAGI-SUGENO
// ---------------------------------------------
tmp1=zeros(npoints,ninputs);
tmp2=zeros(npoints,nrules);
for rul=1:nrules,
	for inp=1:ninputs,
		idx1=abs(fls.rule(rul,inp));
		idx2=sign(fls.rule(rul,inp));
		// EVALUATE MEMBER FUNCTION
		if (idx1>0) then			
			tmp1(:,inp)=mfeval(x(:,inp),fls.input(inp).mf(idx1).type,fls.input(inp).mf(idx1).par);
			// DO COMPLEMENT IF IS REQUIRED
			if (idx2<0) then
				tmp1(:,inp)=complement(tmp1(:,inp),fls.Comp,fls.CompPar);
			end
		else
			tmp1(:,inp)=fls.rule(rul,$-1)*ones(npoints,1);
		end
	end
	// DO S-NORM OR T-NORM
	if (fls.rule(rul,$-1)==0) then
		// S-NORM
		tmp2(:,rul)=snorm(tmp1,fls.SNorm,fls.SNormPar);
	else
		// T-NORM
		tmp2(:,rul)=tnorm(tmp1,fls.TNorm,fls.TNormPar);
	end
end

// ---------------------------------------------
// COMPUTE THE THEN PART FOR ALL INPUT PAIRS
// ---------------------------------------------
if (fls.type=="m") then
	// MAMDANI
	// THIS ALGORITH IS MORE INTENSIVE THAN TAKAGI-SUGENO
	ly=zeros(npoints,noutputs);
	ImpMethod=fls.ImpMethod;
	AggMethod=fls.AggMethod;
	defuzzMethod=fls.defuzzMethod;

	if ((ImpMethod~="min")&(ImpMethod~="prod")) then
		error("Unknow implication method.");
	end

	if ((AggMethod~="max")&(AggMethod~="sum")&(AggMethod~="asum")) then
		error("Unknow implication method.");
	end

	for ip=1:npoints,
		for outp=1:noutputs,
			sp1=fls.output(outp).range;
			dom1=linspace(sp1(1),sp1(2),npev(outp))';
			y1=zeros(npev(outp),1);
			for rul=1:nrules,
				idx1=abs(fls.rule(rul,ninputs+outp));
				idx2=sign(fls.rule(rul,ninputs+outp));
				// EVALUATE MEMBER FUNCTION
				if (idx1>0) then
					y2=mfeval(dom1,fls.output(outp).mf(idx1).type,fls.output(outp).mf(idx1).par);
					// DO COMPLEMENT IF IS REQUIRED
					if (idx2<0) then
						y2=complement(y1,fls.Comp,fls.CompPar);
					end
					// MAKE IMPLICATION
					if (ImpMethod=="prod") then
						y2=y2*tmp2(ip,rul);
					else
						y2=min(y2,tmp2(ip,rul));
					end					
				end
				// MAKE AGGERGATION
				if (AggMethod=="max") then
					y1=max(y1,y2);
				elseif (AggMethod=="sum") then
					y1=y1+y2;
				else
					y1=snorm([y1 y2],"asum");
				end
			end
			// DEFUZZIFICATION
			ly(ip,outp)=defuzzm(dom1,y1,defuzzMethod);
		end
	end
	
else
	// TAKAGI-SUGENO
	if ((fls.defuzzMethod~="waver")&(fls.defuzzMethod~="wsum")) then
		error("Unknow defuzzyfication method.");
	end

	ly=zeros(npoints,noutputs);
	for rul=1:nrules,		
		for outp=1:noutputs,
			idx1=abs(fls.rule(rul,outp+ninputs));
			if (idx1>0) then
				tmp3=mfeval(x,fls.output(outp).mf(idx1).type,fls.output(outp).mf(idx1).par);
				ly(:,outp)=ly(:,outp)+tmp2(:,rul)*fls.rule(rul,$).*tmp3;				
			end
		end
	end

	// DEFUZZIFICATION METHOD IS WAVER ? 
	if (fls.defuzzMethod=="waver") then
		// THIS OPERATION CAN PRODUCE A 1/0 !
		// CHANGE FLOATING POINT EXCEPTION MODE
		ieee_mode=ieee();ieee(2);
		// IMPORTANT:
		// IF ANY OUTPUT DOESN'T TAKE IN COUNT IN ANY RULE, THEN THE RESULTING OUTPUT 
		// COLUMN IN ly() HAVE A %inf IN ALL CELLS
		dowarning=%f;
		for outp=1:noutputs,
			tmp3=zeros(npoints,1);
			for rul=1:nrules,
				if (fls.rule(rul,ninputs+outp)~=0) then
					tmp3=tmp3+tmp2(:,rul)*fls.rule(rul,$);
				end				
			end

			if (~dowarning) then
				somecero=find(tmp3==0,1);
				if (somecero~=[]) then
					dowarning=%t;
				end
			end
			
			ly(:,outp)=ly(:,outp)./tmp3;

		end
		// DISPLAY A WARNING IF SOME RULE WAS UNFIRED FOR AT LEAST ONE PAIR OF INPUTS (1/0)
		if (dowarning) then
			warning("For some or all pair of inputs, at least one rule was unfired!");
		end
		// RESTORE FLOATING POINT EXCEPTION MODE
		ieee(ieee_mode);
	end
end

endfunction

