/* -----------------------------------------------------------------------
 * SCILAB INTERFACE sciFLT
 * -----------------------------------------------------------------------
 * This file is part of sciFLT ( Scilab Fuzzy Logic Toolbox )
 * Copyright (C) 2004 Jaime Urzua Grez
 * mailto:jaime_urzua@yahoo.com
 * Toolbox Revision 0.2 -- 2004/10/12
 * -----------------------------------------------------------------------
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * -----------------------------------------------------------------------
 * CHANGES :
 * 2004-10-06 Change S-Norm and T-Norm call.
 * -----------------------------------------------------------------------
 */

#include "stack-c.h"
#include <string.h>
#include "sciFLT.h"

/* ---------------------------------------------------------------- */
/* INTERFACE TO ANY REGISTERED MEMBER FUNCTION                      */
/* ---------------------------------------------------------------- */
int inter_anymf(fname)
	char *fname;
{
	int mx,nx,lx,mp,np,lp,ly,ierr,one;
	CheckRhs(2,2);
	CheckLhs(1,1);
	GetRhsVar(1,"d",&mx,&nx,&lx);
	GetRhsVar(2,"d",&mp,&np,&lp);
	if (mp!=1) {
		Scierror(999,"This member function need a row vector parameters.");
		ierr=1;
	} else {
		if ((strncmp(fname,"linear",6)==0) | (strncmp(fname,"constant",8)==0)) {
			one=1;
		} else {
			one=nx;
		}
		CreateVar(3,"d",&mx,&one,&ly);
		C2F(mfeval)(fname,stk(lx),&mx,&nx,stk(lp),&np,stk(ly),&ierr);		
	}

	if (ierr==0) {
		LhsVar(1)=3;
	}

	return 0;
}


/* ---------------------------------------------------------------- */
/* INTERFACE TO mfeval                                              */
/* ---------------------------------------------------------------- */
int inter_mfeval(fname)
	char *fname;
{
	int mx,nx,lx,mmf,nmf,lmf,mp,np,lp,ly,ierr,one;
	CheckRhs(3,3);
	CheckLhs(1,1);
	GetRhsVar(1,"d",&mx,&nx,&lx);
	GetRhsVar(2,"c",&mmf,&nmf,&lmf);
	GetRhsVar(3,"d",&mp,&np,&lp);
	if (mp!=1) {
		Scierror(999,"This member function need a row vector parameters.");
		ierr=1;
	} else {
		if ((strncmp(cstk(lmf),"linear",6)==0) | (strncmp(cstk(lmf),"constant",8)==0)) {
			one=1;
		} else {
			one=nx;
		}
		CreateVar(4,"d",&mx,&one,&ly);
		C2F(mfeval)(cstk(lmf),stk(lx),&mx,&nx,stk(lp),&np,stk(ly),&ierr);
	}

	if (ierr==0) {
		LhsVar(1)=4;
	}

	return 0;
}

/* ---------------------------------------------------------------- */
/* INTERFACE TO complement                                          */
/* ---------------------------------------------------------------- */
int inter_complement(fname)
	char *fname;
{
	int mx,nx,lx,mc,nc,lc,extra,ly,icero=0,mp,np,lp,ierr;
	double dcero=0.0;
	CheckRhs(2,3);
	CheckLhs(1,1);
	GetRhsVar(1,"d",&mx,&nx,&lx);
	GetRhsVar(2,"c",&mc,&nc,&lc);
	extra=0;
	if ((strncmp(cstk(lc),"sugeno",6)==0) | (strncmp(cstk(lc),"yager",5)==0)) {
		extra =1;
	}
	
	if (extra==0) {
		CreateVar(3,"d",&mx,&nx,&ly);
		C2F(complement)(cstk(lc),stk(lx),&mx,&nx,&dcero,&icero,stk(ly),&ierr);
	} else {
		mp=0;
		if (Rhs==3) {
			GetRhsVar(3,"d",&mp,&np,&lp);
		}

		if (mp!=1) {
			Scierror(999,"This complement class need a row vector of parameters.");
			ierr=1;
		} else {
			CreateVar(4,"d",&mx,&nx,&ly);
			C2F(complement)(cstk(lc),stk(lx),&mx,&nx,stk(lp),&np,stk(ly),&ierr);
		}
	}

	if (ierr==0) {
		LhsVar(1)=3+extra;
	}
	return 0;
}

/* ---------------------------------------------------------------- */
/* INTERFACE TO tnorm                                               */
/* ---------------------------------------------------------------- */
int inter_tnorm(fname)
	char *fname;
{
	int mx,nx,lx,mc,nc,lc,extra,ly,icero=0,one=1,mp,np,lp,ierr;
	double dcero=0.0;
	CheckRhs(2,3);
	CheckLhs(1,1);
	GetRhsVar(1,"d",&mx,&nx,&lx);
	GetRhsVar(2,"c",&mc,&nc,&lc);
	extra=0;
	if ((strncmp(cstk(lc),"dubois",6)==0) | (strncmp(cstk(lc),"yager",5)==0)) {
		extra =1;
	}
	
	if (extra==0) {
		CreateVar(3,"d",&mx,&one,&ly);
		C2F(ctnorm)(cstk(lc),stk(lx),&mx,&nx,&dcero,&icero,stk(ly),&ierr);
	} else {
		mp=0;
		if (Rhs==3) {
			GetRhsVar(3,"d",&mp,&np,&lp);
		}

		if (mp!=1) {
			Scierror(999,"This t-norm class need a row vector of parameters.");
			ierr=1;
		} else {
			CreateVar(4,"d",&mx,&one,&ly);
			C2F(ctnorm)(cstk(lc),stk(lx),&mx,&nx,stk(lp),&np,stk(ly),&ierr);
		}
	}

	if (ierr==0) {
		LhsVar(1)=3+extra;
	}
	return 0;
}

/* ---------------------------------------------------------------- */
/* INTERFACE TO snorm                                               */
/* ---------------------------------------------------------------- */
int inter_snorm(fname)
	char *fname;
{
	int mx,nx,lx,mc,nc,lc,extra,ly,icero=0,one=1,mp,np,lp,ierr;
	double dcero=0.0;
	CheckRhs(2,3);
	CheckLhs(1,1);
	GetRhsVar(1,"d",&mx,&nx,&lx);
	GetRhsVar(2,"c",&mc,&nc,&lc);
	extra=0;
	if ((strncmp(cstk(lc),"dubois",6)==0) | (strncmp(cstk(lc),"yager",5)==0)) {
		extra =1;
	}
	
	if (extra==0) {
		CreateVar(3,"d",&mx,&one,&ly);
		C2F(csnorm)(cstk(lc),stk(lx),&mx,&nx,&dcero,&icero,stk(ly),&ierr);
	} else {
		mp=0;
		if (Rhs==3) {
			GetRhsVar(3,"d",&mp,&np,&lp);
		}

		if (mp!=1) {
			Scierror(999,"This s-norm class need a row vector of parameters.");
			ierr=1;
		} else {
			CreateVar(4,"d",&mx,&one,&ly);
			C2F(csnorm)(cstk(lc),stk(lx),&mx,&nx,stk(lp),&np,stk(ly),&ierr);
		}
	}

	if (ierr==0) {
		LhsVar(1)=3+extra;
	}
	return 0;
}

/* ---------------------------------------------------------------- */
/* INTERFACE TO defuzzm                                             */
/* ---------------------------------------------------------------- */
int inter_defuzzm(fname)
	char *fname;
{
	int mx,nx,lx;
	int my,ny,ly;
	int mmethod,nmethod,lmethod;
	int ione=1,lout;
	int ierr;
	CheckRhs(3,3);
	CheckLhs(1,1);
	GetRhsVar(1,"d",&mx,&nx,&lx);
	GetRhsVar(2,"d",&my,&ny,&ly);
	GetRhsVar(3,"c",&mmethod,&nmethod,&lmethod);
	if (mx!=my) {
		Scierror(999,"Column vector must have the same size!");
		ierr=1;
	} else {
		if ((nx!=1)|(ny!=1)) {
			Scierror(999,"The first two parameters must be a column vector");
			ierr=1;
		} else {
			CreateVar(4,"d",&ione,&ione,&lout);
			C2F(defuzzm)(cstk(lmethod),stk(lx),&mx,stk(ly),stk(lout),&ierr);
		}
	}	

	if (ierr==0) {
		LhsVar(1)=4;
	}
	return (0);
}

/* ---------------------------------------------------------------- */
/* INTERFACE TO flsencode                                           */
/* ---------------------------------------------------------------- */
int inter_flsencode(fname)
	char *fname;
{
	CheckRhs(1,1);
	CheckLhs(7,7);
	/* flsencode routine have all */
	flsencode(0);
	return (0);
}

/* ---------------------------------------------------------------- */
/* INTERFACE TO evalfls                                             */
/* ---------------------------------------------------------------- */
int inter_evalfls(fname)
	char *fname;
{
	CheckRhs(2,3);
	CheckLhs(1,2);
	/* flsencode routine have all */
	flsencode(1);
	return (0);
}


