c -----------------------------------------------------------------------
c Member functions
c ----------------------------------------------------------------------- 
c This file is part of sciFLT ( Scilab Fuzzy Logic Toolbox )
c Copyright (C) 2004 Jaime Urzua Grez
c mailto:jaime_urzua@yahoo.com
c Toolbox Revision 0.2 -- 2004/10/12
c -----------------------------------------------------------------------
c This program is free software; you can redistribute it and/or modify
c it under the terms of the GNU General Public License as published by
c the Free Software Foundation; either version 2 of the License, or
c (at your option) any later version.
c -----------------------------------------------------------------------

c **************************************************       
c TRIANGULAR MEMBER FUNCTION
c **************************************************       
       subroutine trimf(x,m,n,par,y,ierr)
       integer m,n,ierr
       double precision x(m,n),par(*),y(m,n)

       integer i,j
       double precision a,b,c,tmp1,tmp2

       a=par(1)
       b=par(2)
       c=par(3)
       tmp1=b-a
       tmp2=c-b

c CHECK PARAMETERS
       if ((tmp1.le.0.0D0).or.(tmp2.le.0.0D0)) then
        call erro("Invalid parameters, trimf need [a,b,c] with a<b<c.")
        ierr=1
        goto 9999
       end if

c COMPUTE MEMBER FUNCTION
       do 20, j=1, n
        do 10, i=1, m
         if ((x(i,j).le.a).or.(x(i,j).ge.c)) then
          y(i,j)=0.0D0
         else if (x(i,j).lt.b) then
          y(i,j)=(x(i,j)-a)/tmp1
         else if (x(i,j).eq.b) then
          y(i,j)=1.0D0
         else
          y(i,j)=(c-x(i,j))/tmp2
         end if
10      continue
20     continue
       ierr=0
    
9999   return
       end

c **************************************************       
c TRAPEZOIDAL MEMBER FUNCTION
c **************************************************       
       subroutine trapmf(x,m,n,par,y,ierr)
       integer m,n,ierr
       double precision x(m,n),par(*),y(m,n)

       integer i,j
       double precision a,b,c,d,tmp1,tmp2

       a=par(1)
       b=par(2)
       c=par(3)
       d=par(4)
       tmp1=b-a
       tmp2=d-c

c CHECK PARAMETERS
       if ((tmp1.le.0.0D0).or.(tmp2.le.0.0D0).or.(c.lt.b)) then
        call erro(
     $"Invalid parameters, trapmf need [a,b,c,d] with a<b<=c<d.")
        ierr=1
        goto 9999
       end if

c COMPUTE MEMBER FUNCTION
       do 20, j=1, n
        do 10, i=1, m
         if ((x(i,j).le.a).or.(x(i,j).ge.d)) then
          y(i,j)=0.0D0
         else if (x(i,j).lt.b) then
          y(i,j)=(x(i,j)-a)/tmp1
         else if ((x(i,j).ge.b).and.(x(i,j).le.c)) then
          y(i,j)=1.0D0
         else
          y(i,j)=(d-x(i,j))/tmp2
         end if
10      continue
20     continue
       ierr=0

9999   return
       end

c **************************************************
c GAUSSIAN MEMBER FUNCTION
c **************************************************
       subroutine gaussmf(x,m,n,par,y,ierr)
       integer m,n,ierr
       double precision x(m,n),par(*),y(m,n)

       integer i,j
       double precision a,b,tmp1,tmp2

       a=par(1)
       b=par(2)
       tmp1=b*b

c CHECK PARAMETERS
       if (b.eq.0.0D0) then
        call erro("Invalid parameters, gaussmf need [a,b] with b<>0.")
        ierr=1
        goto 9999
       end if

c COMPUTE MEMBER FUNCTION
       do 20, j=1, n
        do 10, i=1, m
         tmp2=(x(i,j)-a)**2         
         y(i,j)=exp(-tmp2/tmp1)
10      continue
20     continue
       ierr=0
    
9999   return
       end

c **************************************************
c EXTENDED GAUSSIAN MEMBER FUNCTION
c **************************************************
       subroutine gauss2mf(x,m,n,par,y,ierr)
       integer m,n,ierr
       double precision x(m,n),par(*),y(m,n)

       integer i,j
       double precision a,b,c,d,tmp1,tmp2,tmp3,tmp4

       a=par(1)
       b=par(2)
       c=par(3)
       d=par(4)
       tmp1=b*b
       tmp2=d*d

c CHECK PARAMETERS
       if ((b.eq.0.0D0).or.(d.eq.0.0D0)) then
        call erro(
     $"Invalid parameters, gaussmf need [a,b,c,d] with b<>0, d<>0.")
        ierr=1
        goto 9999
       end if

c COMPUTE MEMBER FUNCTION
       do 20, j=1, n
        do 10, i=1, m
         if (x(i,j).le.a) then
          tmp3=(x(i,j)-a)**2
          tmp3=exp(-tmp3/tmp1)
         else
          tmp3=1.0D0
         end if
         
         if (x(i,j).ge.c) then
          tmp4=(x(i,j)-c)**2
          tmp4=exp(-tmp4/tmp2)
         else
          tmp4=1.0D0
         end if
         
         y(i,j)=tmp3*tmp4
10      continue
20     continue
       ierr=0

9999   return
       end

c **************************************************
c SIGMOIDAL MEMBER FUNCTION
c **************************************************
       subroutine sigmf(x,m,n,par,y,ierr)
       integer m,n,ierr
       double precision x(m,n),par(*),y(m,n)

       integer i,j
       double precision a,b,tmp1

       a=par(1)
       b=par(2)

c COMPUTE MEMBER FUNCTION
       do 20, j=1, n
        do 10, i=1, m
         tmp1=par(1)*(x(i,j)-par(2))
         y(i,j)=1.0D0/(1.0D0+exp(-tmp1))
10      continue
20     continue
       ierr=0   

9999   return
       end

c **************************************************
c PRODUCT OF TWO SIGMOIDAL MEMBER FUNCTION
c **************************************************
       subroutine psigmf(x,m,n,par,y,ierr)
       integer m,n,ierr
       double precision x(m,n),par(*),y(m,n)

       integer i,j
       double precision a,b,c,d,tmp1,tmp2

c INITIALIZE SOME INTERNAL
       a=par(1)
       b=par(2)
       c=par(3)
       d=par(4)

c COMPUTE MEMBER FUNCTION
       do 20, j=1, n
        do 10, i=1, m
         tmp1=par(1)*(x(i,j)-par(2))
         tmp1=1.0D0/(1.0D0+exp(-tmp1))
         tmp2=par(3)*(x(i,j)-par(4))
         tmp2=1.0D0/(1+0D0+exp(-tmp2))         
         y(i,j)=tmp1*tmp2
10      continue
20     continue
       ierr=0
    
9999   return
       end

c **************************************************
c DIFFERENCE OF TWO SIGMOIDAL MEMBER FUNCTION
c **************************************************
       subroutine dsigmf(x,m,n,par,y,ierr)
       integer m,n,ierr
       double precision x(m,n),par(*),y(m,n)

       integer i,j
       double precision a,b,c,d,tmp1,tmp2

       a=par(1)
       b=par(2)
       c=par(3)
       d=par(4)

c COMPUTE MEMBER FUNCTION
       do 20, j=1, n
        do 10, i=1, m
          tmp1=par(1)*(x(i,j)-par(2))
          tmp1=1.0D0/(1.0D0+exp(-tmp1))
          tmp2=par(3)*(x(i,j)-par(4))
          tmp2=1.0D0/(1.0D0+exp(-tmp2))
          y(i,j)=abs(tmp1-tmp2)
10      continue
20     continue
       ierr=0
    
9999   return
       end

c **************************************************
c GENERALIZED BELL MEMBER FUNCTION
c **************************************************
       subroutine gbellmf(x,m,n,par,y,ierr)
       integer m,n,ierr
       double precision x(m,n),par(*),y(m,n)

       integer i,j
       double precision a,b,c,tmp1

       a=par(1)
       b=par(2)
       c=par(3)

c CHECK PARAMETERS
       if (b.eq.0.D0) then
        call erro(
     $"Invalid parameters, gbellmf need [a,b,c] with b<>0.")
        ierr=1
        goto 9999
       end if

c COMPUTE MEMBER FUNCTION
       do 20, j=1, n
        do 10, i=1, m
         if (par(3).eq.0.D0) then
          y(i,j)=0.5D0
         elseif (par(3).lt.0.0D0) then
          y(i,j)=0.0D0
         else
          tmp1=(x(i,j)-a)/b
          if (tmp1.lt.0.0D0) then
           tmp1=-tmp1
          endif
          tmp1=tmp1**par(3)
          y(i,j)=1.0D0/(1.0D0+tmp1)
         end if
10      continue
20     continue
       ierr=0
    
9999   return
       end

c **************************************************
c PI-SHAPED MEMBER FUNCTION
c **************************************************
       subroutine pimf(x,m,n,par,y,ierr)
       integer m,n,ierr
       double precision x(m,n),par(*),y(m,n)

       integer i,j
       double precision a(2),b(2),tmp1,tmp2

       a(1)=par(1)
       a(2)=par(2)
       b(1)=par(3)
       b(2)=par(4)

c COMPUTE MEMBER FUNCTION
       do 20, j=1, n
        do 10, i=1, m
c        TAKE MUCH TIME  OPTMIZATION ?
         call smf(x(i,j),1,1,a,tmp1,ierr)
         call zmf(x(i,j),1,1,b,tmp2,ierr)
         y(i,j)=tmp1*tmp2
10      continue
20     continue       

9999   return
       end       

c **************************************************
c S-SHAPED MEMBER FUNCTION
c **************************************************
       subroutine smf(x,m,n,par,y,ierr)
       integer m,n,ierr
       double precision x(m,n),par(*),y(m,n)

       integer i,j
       double precision a,b,tmp1,tmp2

       a=par(1)
       b=par(2)
       tmp1=(a+b)/2.0D0
       tmp2=b-a

c COMPUTE MEMBER FUNCTION
       if (a.ge.b) then
        do 20, j=1, n
         do 10, i=1, m
          if (x(i,j).ge.tmp1) then
           y(i,j)=1.0D0
          else
           y(i,j)=0.0D0
          end if
10       continue
20      continue
       else
        do 40, j=1, n
         do 30, i=1, m
          if (x(i,j).le.a) then
           y(i,j)=0.0D0
          else if (x(i,j).le.tmp1) then
           y(i,j)=2.0D0*((x(i,j)-a)/tmp2)**2
          else if (x(i,j).le.b) then
           y(i,j)=1.0D0-2.0D0*((b-x(i,j))/tmp2)**2
          else
           y(i,j)=1.0D0
          end if
30       continue
40      continue
       end if
       ierr=0
    
9999   return
       end

c **************************************************
c Z-SHAPED MEMBER FUNCTION
c **************************************************
       subroutine zmf(x,m,n,par,y,ierr)
       integer m,n,ierr
       double precision x(m,n),par(*),y(m,n)

       integer i,j
       double precision a,b,tmp1,tmp2

       a=par(1)
       b=par(2)
       tmp1=(a+b)/2.0D0
       tmp2=b-a

c COMPUTE MEMBER FUNCTION
       if (a.ge.b) then
        do 20, j=1, n
         do 10, i=1, m
          if (x(i,j).le.tmp1) then
           y(i,j)=1.0D0
          else
           y(i,j)=0.0D0
          end if
10       continue
20      continue
       else
        do 40, j=1, n
         do 30, i=1, m
          if (x(i,j).le.a) then
           y(i,j)=1.0D0
          else if (x(i,j).le.tmp1) then
           y(i,j)=1.0D0-2.0D0*((x(i,j)-a)/tmp2)**2
          else if (x(i,j).le.b) then
           y(i,j)=2.0D0*((b-x(i,j))/tmp2)**2
          else
           y(i,j)=0.0D0
          end if
30       continue
40      continue
       end if
       ierr=0

9999   return
       end

c **************************************************
c CONSTANT MEMBER FUNCTION
c **************************************************
       subroutine constant(x,m,n,par,y,ierr)
       integer m,n,ierr
       double precision x(m,n),par(*),y(m)

       integer i

c COMPUTE MEMBER FUNCTION
       do 10, i=1, m
        y(i)=par(1)
10     continue
       ierr=0

9999   return
       end       

c **************************************************
c LINEAR MEMBER FUNCTION
c **************************************************
       subroutine linear(x,m,n,par,y,ierr)
       integer m,n,ierr
       double precision x(m,n),par(*),y(m)

       integer i,j
       double precision tmp1

c COMPUTE MEMBER FUNCTION
       tmp1=par(n+1)
       do 10, i=1, m
        y(i)=tmp1
10     continue
        
       do 30, j=1, n
        tmp1=par(j)
        do 20, i=1, m
         y(i)=y(i)+x(i,j)*tmp1
20      continue
30     continue
       ierr=0 


9999   return
       end

c **************************************************
c EVALUATE MEMBER FUNCTION
c **************************************************
      subroutine mfeval(mfname,x,m,n,par,npar,y,ierr)
      character*(*) mfname
      integer m,n,npar,ierr
      double precision x(m,n),par(*),y(m,n)

c TRIANGULAR MEMBER FUNCTION
      if (mfname(1:5).eq."trimf") then
       if (npar.ne.3) then
        call erro("trimf need 3 parameters in a row vector.")
        ierr=1
       else
        call trimf(x,m,n,par,y,ierr)
       end if

c TRAPEZOIDAL MEMBER FUNCTION
      else if (mfname(1:6).eq."trapmf") then
       if (npar.ne.4) then
        call erro("trapmf need 4 parameters in a row vector.")
        ierr=1
       else
        call trapmf(x,m,n,par,y,ierr)
       end if

c GAUSSIAN MEMBER FUNCTION
      else if (mfname(1:7).eq."gaussmf") then       
       if (npar.ne.2) then
        call erro("gaussmf need 2 parameters in a row vector.")
        ierr=1
       else
        call gaussmf(x,m,n,par,y,ierr)
       end if

c EXTENDED GAUSSIAN MEMBER FUNCTION
      else if (mfname(1:8).eq."gauss2mf") then
       if (npar.ne.4) then
        call erro("gauss2mf need 4 parameters in a row vector.")
        ierr=1
       else
        call gauss2mf(x,m,n,par,y,ierr)
       end if
       
c SIGMOIDAL MEMBER FUNCTION
      else if (mfname(1:5).eq."sigmf") then
       if (npar.ne.2) then
        call erro("sigmf need 2 parameters in a row vector.")
        ierr=1
       else
        call sigmf(x,m,n,par,y,ierr)
       end if

c PRODUCT OF TWO SIGMOIDAL MEMBER FUNCTION
      else if (mfname(1:6).eq."psigmf") then
       if (npar.ne.4) then
        call erro("psigmf need 4 parameters in a row vector.")
        ierr=1
       else
        call psigmf(x,m,n,par,y,ierr)
       end if
       
c DIFFERENCE OF TWO SIGMOIDAL MEMBER FUNCTION
      else if (mfname(1:6).eq."dsigmf") then
       if (npar.ne.4) then
        call erro("dsigmf need 4 parameters in a row vector.")
        ierr=1
       else
        call dsigmf(x,m,n,par,y,ierr)
       end if
       
c GENERALIZED BELL MEMBER FUNCTION
      else if (mfname(1:7).eq."gbellmf") then
       if (npar.ne.3) then
        call erro("gbellmf need 3 parameters in a row vector.")
        ierr=1
       else
        call gbellmf(x,m,n,par,y,ierr)
       end if

c Pi-Shaped BELL MEMBER FUNCTION
      else if (mfname(1:4).eq."pimf") then
       if (npar.ne.4) then
        call erro("pimf need 4 parameters in a row vector.")
        ierr=1
       else
        call pimf(x,m,n,par,y,ierr)
       end if

c S-Shaped BELL MEMBER FUNCTION
      else if (mfname(1:3).eq."smf") then
       if (npar.ne.2) then
        call erro("smf need 2 parameters in a row vector.")
        ierr=1
       else
        call smf(x,m,n,par,y,ierr)
       end if

c Z-Shaped BELL MEMBER FUNCTION
      else if (mfname(1:3).eq."zmf") then
       if (npar.ne.2) then
        call erro("zmf need 2 parameters in a row vector.")
        ierr=1
       else
        call zmf(x,m,n,par,y,ierr)
       end if

c CONSTANT MEMBER FUNCTION
      else if (mfname(1:8).eq."constant") then
       if (npar.ne.1) then
        call erro("constant need 1 parameters in a row vector.")
        ierr=1
       else
        call constant(x,m,n,par,y,ierr)
       end if

c LINEAR MEMBER FUNCTION
      else if (mfname(1:6).eq."linear") then
       if (npar.ne.(n+1)) then
        call erro("incorrect number of parameters.")
        ierr=1
       else
        call linear(x,m,n,par,y,ierr)
       end if
       
c UNKNOW MEMBER FUNCTION
      else
       call erro("Unknow Member Function Type.")
       ierr=1
      end if
      
9999  return
      end


c **************************************************
c EVALUATE MEMBER FUNCTION USING ID
c **************************************************
      subroutine mfeval2(mfid,x,m,n,par,y,ierr)
      integer mfid,m,n,ierr
      double precision x(m,n),par(*),y(m,n)
       
c TRIANGULAR MEMBER FUNCTION
      if (mfid.eq.1) then       
       call trimf(x,m,n,par,y,ierr)

c TRAPEZOIDAL MEMBER FUNCTION
      else if (mfid.eq.2) then
       call trapmf(x,m,n,par,y,ierr)

c GAUSSIAN MEMBER FUNCTION
      else if (mfid.eq.3) then       
       call gaussmf(x,m,n,par,y,ierr)
       
c EXTENDED GAUSSIAN MEMBER FUNCTION
      else if (mfid.eq.4) then
       call gauss2mf(x,m,n,par,y,ierr)
       
c SIGMOIDAL MEMBER FUNCTION
      else if (mfid.eq.5) then
       call sigmf(x,m,n,par,y,ierr)
       
c PRODUCT OF TWO SIGMOIDAL MEMBER FUNCTION
      else if (mfid.eq.6) then
       call psigmf(x,m,n,par,y,ierr)
       
c DIFFERENCE OF TWO SIGMOIDAL MEMBER FUNCTION
      else if (mfid.eq.7) then
       call dsigmf(x,m,n,par,y,ierr)
     
c GENERALIZED BELL MEMBER FUNCTION
      else if (mfid.eq.8) then
       call gbellmf(x,m,n,par,y,ierr)

c Pi-Shaped BELL MEMBER FUNCTION
      else if (mfid.eq.9) then
       call pimf(x,m,n,par,y,ierr)

c S-Shaped BELL MEMBER FUNCTION
      else if (mfid.eq.10) then
       call smf(x,m,n,par,y,ierr)

c Z-Shaped BELL MEMBER FUNCTION
      else if (mfid.eq.11) then
       call zmf(x,m,n,par,y,ierr)

c CONSTANT
      else if (mfid.eq.12) then
       call constant(x,m,n,par,y,ierr)

c LINEAR
      else if (mfid.eq.13) then
       call linear(x,m,n,par,y,ierr)
      
c UNKNOW MEMBER FUNCTION
      else
       call erro("Unknow Member Function Type.")
       ierr=1
      end if
      
9999  return
      end

c **************************************************
c USED WITH SCICOS
c **************************************************
      subroutine smfeval(flag,nevprt,t,xd,x,nx,z,nz,tvec,ntvec,rpar,
     $nrpar,ipar,nipar,u,nu,y,ny)
      double precision t,xd(*),x(*),z(*),tvec(*),rpar(*),u(*),y(*)
      integer flag,nevprt,nx,nz,ntvec,nrpar,ipar(*),nipar,nu,ny
      integer ierr

      call mfeval2(ipar(1),u,nu,1,rpar,y,ierr)      
 
      return
      end 
