/* -----------------------------------------------------------------------
 * This file is part of sciFLT ( Scilab Fuzzy Logic Toolbox )
 * Copyright (C) 2004 Jaime Urzua Grez
 * mailto:jaime_urzua@yahoo.com
 * Toolbox Revision 0.2 -- 2004/10/12
 * -----------------------------------------------------------------------
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * -----------------------------------------------------------------------
 * CHANGES :
 * 2004-10-06 Change S-Norm and T-Norm call
 * -----------------------------------------------------------------------
 */

#define SCIFLT_REV "0.2"
#define SCIFLT_MAXNPEV 1001

/* FLS INTERNAL CODIFICATION */
extern int flsencode(int mode);

/* FLS EVALUATION */
extern int C2F(flsengine) __PARAMS ((double *x, int *mid1, double *mid2, double *mew, int *mrule, double *mdomo, double *mpari, double *mparo, int *nparo, int *npev, int *ninputs, int *noutputs, int *nrules, int *npoints, double *tmp1, double *tmp2, double *tmp3, double *tmp4, int *maxnpev, double *y, int *ierr));

/* COMPLEMENT */
extern int C2F(complement) __PARAMS((char *class1, double *x, int *m, int *n, double *par, int *npar, double *y, int *ierr));
extern int C2F(complement2) __PARAMS((int *classid, double *x, int *m, int *n, double *par, double *y, int *ierr));
extern int C2F(scomplement) __PARAMS((int *flag, int*nevprt, double *t, double *xd, double *x, int *nx, double *z, int *nz, double *tvec, int *ntvec, double *rpar, int *nrpar, int *ipar, int *nipar, double *u, int *nu, double *y, int *ny));

/* S-NORM */
extern int C2F(csnorm) __PARAMS((char *class1, double *x, int *m, int *n, double *par, int *npar, double *y, int *ierr));
extern int C2F(csnorm2) __PARAMS((int *classid, double *x, int *m, int *n, double *par, double *y, int *ierr));
extern int C2F(ssnorm) __PARAMS((int *flag, int*nevprt, double *t, double *xd, double *x, int *nx, double *z, int *nz, double *tvec, int *ntvec, double *rpar, int *nrpar, int *ipar, int *nipar, double *u, int *nu, double *y, int *ny));

/* T-NORM */
extern int C2F(ctnorm) __PARAMS((char *class1, double *x, int *m, int *n, double *par, int *npar, double *y, int *ierr));
extern int C2F(ctnorm2) __PARAMS((int *classid, double *x, int *m, int *n, double *par, double *y, int *ierr));
extern int C2F(stnorm) __PARAMS((int *flag, int*nevprt, double *t, double *xd, double *x, int *nx, double *z, int *nz, double *tvec, int *ntvec, double *rpar, int *nrpar, int *ipar, int *nipar, double *u, int *nu, double *y, int *ny));

/* MEMBER FUNCTIONS */
extern int C2F(trimf) __PARAMS((double *x, int *m, int *n, double *par, double *y, int *ierr));
extern int C2F(trapmf) __PARAMS((double *x, int *m, int *n, double *par, double *y, int *ierr));
extern int C2F(gaussmf) __PARAMS((double *x, int *m, int *n, double *par, double *y, int *ierr));
extern int C2F(gauss2mf) __PARAMS((double *x, int *m, int *n, double *par, double *y, int *ierr));
extern int C2F(sigmf) __PARAMS((double *x, int *m, int *n, double *par, double *y, int *ierr));
extern int C2F(psigmf) __PARAMS((double *x, int *m, int *n, double *par, double *y, int *ierr));
extern int C2F(dsigmf) __PARAMS((double *x, int *m, int *n, double *par, double *y, int *ierr));
extern int C2F(gbellmf) __PARAMS((double *x, int *m, int *n, double *par, double *y, int *ierr));
extern int C2F(pimf) __PARAMS((double *x, int *m, int *n, double *par, double *y, int *ierr));
extern int C2F(smf) __PARAMS((double *x, int *m, int *n, double *par, double *y, int *ierr));
extern int C2F(zmf) __PARAMS((double *x, int *m, int *n, double *par, double *y, int *ierr));
extern int C2F(constant) __PARAMS((double *x, int *m, int *n, double *par, double *y, int *ierr));
extern int C2F(linear) __PARAMS((double *x, int *m, int *n, double *par, double *y, int *ierr));
extern int C2F(mfeval) __PARAMS((char *mfname, double *x, int *m, int *n, double *par, int *npar, double *y, int *ierr));
extern int C2F(mfeval2) __PARAMS((int *mfid, double *x, int *m, int *n, double *par, double *y, int *ierr));
extern int C2F(smfeval) __PARAMS((int *flag, int*nevprt, double *t, double *xd, double *x, int *nx, double *z, int *nz, double *tvec, int *ntvec, double *rpar, int *nrpar, int *ipar, int *nipar, double *u, int *nu, double *y, int *ny));

/* DEFUZZYFICATION  */
extern int C2F(defuzzm) __PARAMS((char *method, double *x, int *m, double *y, double *ou, int *ierr));
extern int C2F(defuzzm2) __PARAMS ((int *methodid, double *x, int *m, double *y, double *ou, int *ierr));

/* UTIL */
extern int C2F(getmw) __PARAMS ((double *rule, int *nrule, int *ninputs,int *noutputs, double *mwe));
extern int C2F(getmr) __PARAMS ((double *rule, int *nrule, int *ninputs,int *noutputs, int *mrule));
extern int sciFLT_doerror(int nerr);

	
